#pragma once
#include <stdarg.h>
#include "mbed.h"

#include "Color565.h"
#include "font_IBM.h"
#include "LCD_ST7735.h"
#include "Accelerometer.h"
#include "Shapes.h"
#include "Ball.h"
#include "Paddle.h"

//#include "DisplayN18.h"




class Game {    
    static const char* LOSE_1;
    static const char* LOSE_2;
    static const char* SPLASH_1;
    static const char* SPLASH_2;
    static const char* SPLASH_3;
    //char buf[256];
    
    //static const int BALL_RADIUS = 3;
    static const int BALL_RADIUS = 6;
    static const int PADDLE_WIDTH = 38;
    static const int PADDLE_HEIGHT = 4;
    static const int PADDLE_SPEED = 4;
    static const int BOUNCE1_SOUND_TICKS = 1;
    static const int BOUNCE2_SOUND_TICKS = 2;
    static const int GRAPH_HEIGHT = 40;
    static const int GRAPH_SPACING = 2;
    static const char I2C_ADDR = 0x1C << 1;


    DigitalIn left;
    DigitalIn right;
    DigitalIn down;
    DigitalIn up;
    DigitalIn square;
    DigitalIn circle; 
    DigitalOut led1;
    DigitalOut led2;
    PwmOut pwm;
    AnalogIn ain;
    I2C i2c;
//    DisplayN18 disp;
    LCD_ST7735 disp;

    Accelerometer accel;
    Timer tWait;    // timer used for tickcounts

    Vector vGravity;
    Ball ball;
    Paddle paddle;

    int paddleX;
    int pwmTicksLeft;
    int lives;
    int graphX;    
    bool mode;
    bool lastUp;
    bool lastDown;
    unsigned short colors[3];

    
/*
    void readRegisters(char address, char* buffer, int len);
    int writeRegister(char address, char value);
    double convert(char* buffer);
    void getXYZ(double& x, double& y, double& z);
*/
    void printDouble(double value, int x, int y);
    
    void drawAxes();
    void drawPoint(int axis, double value);
    void checkGraphReset();
    
    void initialize();
    void initializeBall();
    void initializePaddle();
    
    void drawString(const char* str, int y);
    
    void clearPaddle();
    void drawPaddle();
    void updatePaddle();
    
    int checkTilt();
    void checkButtons();
    void checkCollision();
    void checkPwm();
    void Printf(int x, int y, const char *szFormat, ...);
    void checkLives();
    
    public:
        Game();
        
        void showSplashScreen();
        void tick();
};